@@ -24,10 +24,14 @@
 #include <qpx_mmc.h>
 #include "plextor_features.h"
 
+#define ASDB_SAVE_DEBUG
+#define ASDB_LOAD_DEBUG
+
 int plextor_get_TLA(drive_info* drive) {
 	drive->cmd_clear();
 	drive->cmd[0] = 0xF1;
-	drive->cmd[9] = 0x80;
+	drive->cmd[8] = 0x01;
+	drive->cmd[9] = 0x00;
 	// The Plextor PX-716 does not understand this command....
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x100)))
 	{
@@ -35,7 +39,8 @@ int plextor_get_TLA(drive_info* drive) {
 		drive->cmd_clear();
 		drive->cmd[0] = 0xF1;
 		drive->cmd[1] = 0x01;
-		drive->cmd[9] = 0x80;
+		drive->cmd[8] = 0x01;
+		drive->cmd[9] = 0x00;
 		if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x100) ))
 			{strcpy(drive->TLA,"N/A\0"); return 1;}
 	}
@@ -47,9 +52,9 @@ int plextor_get_TLA(drive_info* drive) {
 int plextor_get_speeds(drive_info* drive) {
 	int sel, max, last;
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_PREC_SPD;
+	drive->cmd[0]=PLEXTOR_PREC_SPD;
 	drive->cmd[9]=0x0A;
-	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,8) ))
+	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x0A) ))
 		{ if (!drive->silent) sperror ("GET_SPEEDS",drive->err);return drive->err;}
 //	sel = (drive->rd_buf[5]<<8) | drive->rd_buf[4];
 //	max = (drive->rd_buf[7]<<8) | drive->rd_buf[6];
@@ -67,7 +72,7 @@ int plextor_get_speeds(drive_info* drive) {
 
 int plextor_get_powerec(drive_info* drive) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE2;
+	drive->cmd[0]=PLEXTOR_MODE2;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=0x00;
 	drive->cmd[9]=0x08;
@@ -85,7 +90,7 @@ int plextor_get_powerec(drive_info* drive) {
 int plextor_set_powerec(drive_info* drive) {
 	printf("\tTurning PoweRec %s\n",drive->plextor.powerec_state ? "ON" : "OFF");
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE2;
+	drive->cmd[0]=PLEXTOR_MODE2;
 	drive->cmd[1]=PLEX_SET_MODE | (drive->plextor.powerec_state?1:0);
 	drive->cmd[2]=0x00;
 	drive->cmd[9]=0x08;
@@ -126,7 +131,7 @@ void plextor_print_silentmode_state(drive_info* drive){
 
 int plextor_get_silentmode(drive_info* drive) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SILENT;
 	drive->cmd[3]=0x04;
@@ -141,7 +146,7 @@ int plextor_get_silentmode(drive_info* drive) {
 
 int plextor_set_silentmode_tray(drive_info* drive, int disc_type, int permanent) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SILENT_TRAY;
 	drive->cmd[3]=disc_type | 2*!!permanent;
@@ -155,7 +160,7 @@ int plextor_set_silentmode_tray(drive_info* drive, int disc_type, int permanent)
 
 int plextor_set_silentmode_disc(drive_info* drive, int disc_type, int permanent) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SILENT_DISC;
 	drive->cmd[3]=disc_type | 2*!!permanent;
@@ -196,7 +201,7 @@ int plextor_set_gigarec(drive_info* drive) {
 // 	printf("  applying gigarec setting... ");
 // 	print_gigarec_value(i);
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_GIGAREC;
 	drive->cmd[3]=(drive->plextor.gigarec?1:0);
@@ -213,7 +218,7 @@ int plextor_set_gigarec(drive_info* drive) {
 
 int plextor_get_gigarec(drive_info* drive) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_GIGAREC;
 	drive->cmd[10]=0x08;
@@ -242,7 +247,7 @@ void print_varirec(drive_info* drive, int disc_type) {
 
 int plextor_set_varirec(drive_info* drive, int disc_type) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_VARIREC;
 	if (disc_type == VARIREC_DVD) {
@@ -272,7 +277,7 @@ int plextor_set_varirec(drive_info* drive, int disc_type) {
 
 int plextor_get_varirec(drive_info* drive, int disc_type) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_VARIREC;
 	drive->cmd[3]=0x02 | disc_type;
@@ -302,7 +307,7 @@ void print_securec_state(drive_info* drive) {
 int plextor_get_securec_state(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[2]=PLEX_MODE_SECUREC;
 	drive->cmd[10]=0x08;
 	if (drive->err = drive->cmd.transport(READ, drive->rd_buf, 8))
@@ -321,7 +326,7 @@ int plextor_set_securec(drive_info* drive, char len, char* passwd)
 {
 	char i;
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_SEND_AUTH;
+	drive->cmd[0]=PLEXTOR_SEND_AUTH;
 	if ((passwd) && (len)) {
 		printf("Turning SecuRec ON\n");
 //		printf("Setting SecuRec passwd to '%s' (len = %d)\n", passwd, len);
@@ -352,7 +357,7 @@ void print_speedread_state(drive_info* drive){
 
 int plextor_set_speedread(drive_info* drive, int state) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_SPDREAD;
 	drive->cmd[3]=!!state;
@@ -367,7 +372,7 @@ int plextor_set_speedread(drive_info* drive, int state) {
 int plextor_get_speedread(drive_info* drive) {
 //	return 1;
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SPDREAD;
 	drive->cmd[3]=0;
@@ -388,7 +393,7 @@ void print_sss_state(drive_info* drive) {
 
 int plextor_get_hidecdr_singlesession(drive_info* drive) {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_SS_HIDE;
 	drive->cmd[9]=0x08;
@@ -403,7 +408,7 @@ int plextor_set_hidecdr_singlesession(drive_info* drive, int hidecdr_state, int
 	if (plextor_get_hidecdr_singlesession(drive)) return 1;
 //	printf("Trying to change SS/HIDE state to %d...\n",2*!!hidecdr_state + !!singlesession_state);
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_SS_HIDE;
 	drive->cmd[3]=2*!!hidecdr_state + !!singlesession_state;
@@ -431,7 +436,7 @@ int plextor_set_singlesession(drive_info* drive, int state) {
 int plextor_get_bitset(drive_info* drive, int disc_type)
 {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_BITSET;
 	drive->cmd[3]=disc_type;
@@ -456,7 +461,7 @@ int plextor_set_bitset(drive_info* drive, int disc_type)
 		default:		printf("PLEXTOR_SET_BITSET: Invalid disc_type"); return 1;
 	}
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_SET_MODE;
 	drive->cmd[2]=PLEX_MODE_BITSET;
 	drive->cmd[3]=disc_type;
@@ -470,7 +475,7 @@ int plextor_set_bitset(drive_info* drive, int disc_type)
 int plextor_get_testwrite_dvdplus(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_MODE;
+	drive->cmd[0]=PLEXTOR_MODE;
 	drive->cmd[1]=PLEX_GET_MODE;
 	drive->cmd[2]=PLEX_MODE_TESTWRITE_DVDPLUS;
 	drive->cmd[10]= 0x08;
@@ -483,7 +488,7 @@ int plextor_get_testwrite_dvdplus(drive_info* drive)
 int plextor_set_testwrite_dvdplus(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_MODE;
+	drive->cmd[0] = PLEXTOR_MODE;
 	drive->cmd[1] = PLEX_SET_MODE;
 	drive->cmd[2] = PLEX_MODE_TESTWRITE_DVDPLUS;
 	drive->cmd[3] = drive->plextor.testwrite_dvdplus;
@@ -498,7 +503,7 @@ int plextor_plexeraser(drive_info* drive)
 	printf("Destucting disc [mode=%02X]... \n",drive->plextor.plexeraser);
 //	return 0;
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_ERASER;
+	drive->cmd[0] = PLEXTOR_PLEXERASER;
 	drive->cmd[1] = 0x06;
 	drive->cmd[2] = drive->plextor.plexeraser;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0) ))
@@ -532,7 +537,7 @@ int plextor_print_autostrategy_state(drive_info* drive)
 int plextor_get_autostrategy(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0]=CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[10]= 0x08;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,8) ))
 		{ if (!drive->silent) sperror ("PLEXTOR_GET_AUTOSTRATEGY",drive->err); return drive->err;}
@@ -544,7 +549,7 @@ int plextor_get_autostrategy(drive_info* drive)
 int plextor_set_autostrategy(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[2] = PLEX_SET_MODE + (drive->astrategy.state & 0x0F);
 	drive->cmd[10]= 0x08;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,8) ))
@@ -558,7 +563,7 @@ int plextor_set_autostrategy(drive_info* drive)
 int plextor_get_autostrategy_db_entry_count(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x02;
 	drive->cmd[10]= 0x08;
 	if ((drive->err=drive->cmd.transport(READ,(void*)&(drive->astrategy),8) ))
@@ -573,7 +578,7 @@ int plextor_get_autostrategy_db(drive_info* drive)//, void* database)
 	int size = 8 + (int)drive->astrategy.dbcnt * 32;
 	int i,j;
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD; // 0xE4
+	drive->cmd[0] = PLEXTOR_AS_RD; // 0xE4
 	drive->cmd[1] = 0x02;
 	drive->cmd[9]= (size >> 8) & 0xFF ;
 	drive->cmd[10]= size & 0xFF ;
@@ -615,6 +620,173 @@ int plextor_get_autostrategy_db(drive_info* drive)//, void* database)
 	return 0;
 }
 
+// EXPERIMENTAL   STRATEGY SAVE
+
+int plextor_get_strategy(drive_info* drive){
+	int cnt,acnt;
+	int i,s,offs;
+	unsigned char *entry;
+	unsigned char *entry_data;
+
+	printf("RETR AS cnt...\n");
+	drive->cmd_clear();
+	drive->cmd[0] = PLEXTOR_AS_RD; // 0xE4
+	drive->cmd[1] = 0x02;
+	drive->cmd[2] = 0x03;
+	drive->cmd[10]= 0x08;
+	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x08) ))
+		{ if (!drive->silent) sperror ("PLEXTOR_GET_STRATEGY",drive->err); return drive->err;}
+	acnt= drive->rd_buf[0];
+	cnt=  drive->rd_buf[6];
+	drive->astrategy.dbcnt = cnt;
+	for (i=0; i<8; i++) printf("%02X ", drive->rd_buf[i]);
+	printf("\nStrategies count: %d\n", drive->astrategy.dbcnt);
+
+	printf("RETR AS data...\n");
+	drive->cmd_clear();
+	drive->cmd[0] = PLEXTOR_AS_RD; // 0xE4
+	drive->cmd[1] = 0x02;
+	drive->cmd[2] = 0x03;
+	drive->cmd[9] = cnt;
+	drive->cmd[10]= 0x10;
+	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,cnt*0x100 + 0x10) ))
+		{ if (!drive->silent) sperror ("PLEXTOR_GET_STRATEGY_DATA",drive->err); return drive->err;}
+
+
+#ifdef ASDB_SAVE_DEBUG
+	printf("DB HDR0:\n");
+	for (i=0; i<8; i++) printf("%02X ", drive->rd_buf[i]); printf("\n");
+	for (s=0; s<cnt; s++) {
+		offs = 8 + s*0x20;
+		for (i=0; i<0x20; i++) printf("%02X ", drive->rd_buf[offs+i]);
+		printf("\n");
+	}
+	offs = 8 + cnt*0x20;
+	printf("DB HDR1:\n");
+	for (i=0; i<8; i++) printf("%02X ", drive->rd_buf[offs+i]); printf("\n");
+	for (s=0; s<cnt; s++) {
+		printf("Strategy #%02d\n", s+1);
+		for(int s1=0; s1<7; s1++) {
+			offs = 0x10 + cnt*0x20 + s*0xE0 + s1*0x20;
+			for (i=0; i<0x20; i++) printf("%02X ", drive->rd_buf[offs+i]);
+			printf("\n");
+		}
+	}
+#endif
+
+	for (s=0; s<cnt; s++) {
+		entry      = (unsigned char*)&(drive->astrategy.entry[s]);
+		entry_data = (unsigned char*)&(drive->astrategy.entry_data[s]);
+
+		offs = 8 + s*0x20;
+		for (i=0; i<0x20; i++) entry[i] = drive->rd_buf[offs+i];
+
+		offs = 0x10 + cnt*0x20 + s*0xE0;
+		for (i=0; i<(0x20*7); i++) entry_data[i] = drive->rd_buf[offs+i];
+	}
+	return 0;
+}
+
+// EXPERIMENTAL   STRATEGY LOAD
+
+int plextor_add_strategy(drive_info* drive){
+	int i, cnt = drive->astrategy.dbcnt;
+	int offs;
+	unsigned char* entry;
+
+	plextor_clear_autostrategy_db(drive);
+	printf("Adding strategy...\n");
+
+// sending strategy headers
+	for (i=0; i<(8+cnt*0x20); i++) drive->rd_buf[i]=0;
+
+	drive->rd_buf[0]= (cnt >> 3) & 0xFF;
+	drive->rd_buf[1]= (cnt*0x20 + 6) & 0xFF;
+	drive->rd_buf[2]=0x02;
+	drive->rd_buf[3]=0x80;
+	drive->rd_buf[6]= cnt;
+	drive->rd_buf[7]=0x20;
+
+	entry = (unsigned char*) &drive->astrategy.entry;
+	for (i=0; i<cnt*0x20; i++) drive->rd_buf[8+i] = entry[i];
+	for (i=0; i<cnt; i++) drive->rd_buf[8+i*0x20] = (unsigned char)(i+1 & 0xFF);
+
+#ifdef ASDB_LOAD_DEBUG
+	printf("DB HDR0:\n");
+	for (i=0; i<8; i++) printf("%02X ", drive->rd_buf[i]); printf("\n");
+	for (int s=0; s<cnt; s++) {
+		offs = 0x8 + s*0x20;
+		for (i=0; i<0x20; i++) printf("%02X ", drive->rd_buf[offs+i]);
+		printf("\n");
+	}
+#endif
+
+	drive->cmd_clear();
+	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
+	drive->cmd[1] = 0x02;
+	drive->cmd[9] = (cnt >> 3) & 0xFF;
+	drive->cmd[10]= (cnt*0x20 + 0x08) & 0xFF;
+	if ((drive->err=drive->cmd.transport(WRITE,drive->rd_buf,cnt*0x20+8) ))
+		{ if (!drive->silent) sperror ("PLEXTOR_ADD_STRATEGY_HDR",drive->err); return drive->err;}
+
+// sending strategy parameters
+	for (i=0; i<(8+cnt*0xE0); i++) drive->rd_buf[i]=0;
+
+	drive->rd_buf[0]= ((cnt*0xE) >> 4) & 0xFF;
+	drive->rd_buf[1]= (cnt*0xE0 + 6) & 0xFF;
+	drive->rd_buf[2]=0x02;
+	drive->rd_buf[3]=0x81;
+	drive->rd_buf[6]= cnt * 7;
+	drive->rd_buf[7]=0x20;
+
+	entry = (unsigned char*) &drive->astrategy.entry_data;
+	for (i=0; i<cnt*0xE0; i++) drive->rd_buf[8+i] = entry[i];
+	for (i=0; i<cnt; i++)
+		for (int s1=0; s1<7; s1++) {
+			drive->rd_buf[i*0xE0+s1*0x20+8] = ((i*7+s1)>>8) & 0xFF;
+			drive->rd_buf[i*0xE0+s1*0x20+9] = (i*7+s1) & 0xFF;
+		}
+
+#ifdef ASDB_LOAD_DEBUG
+	printf("DB HDR1:\n");
+
+
+	for (i=0; i<8; i++) printf("%02X ", drive->rd_buf[i]); printf("\n");
+	for (int s=0; s<cnt; s++) {
+		printf("Strategy #%02d\n", s+1);
+		for(int s1=0; s1<7; s1++) {
+			offs = 0x8 + s*0xE0 + s1*0x20;
+			for (i=0; i<0x20; i++) printf("%02X ", drive->rd_buf[offs+i]);
+			printf("\n");
+		}
+	}
+#endif
+
+	drive->cmd_clear();
+	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
+	drive->cmd[1] = 0x02;
+	drive->cmd[9] = ((cnt*0xE) >> 4) & 0xFF;
+	drive->cmd[10]= (cnt*0xE0 + 0x08) & 0xFF;
+	if ((drive->err=drive->cmd.transport(WRITE,drive->rd_buf,cnt*0xE0+8) ))
+		{ if (!drive->silent) sperror ("PLEXTOR_ADD_STRATEGY_DATA",drive->err); return drive->err;}
+	return 0;
+}
+
+int plextor_clear_autostrategy_db(drive_info* drive) {
+	for (int i=0; i<8; i++) drive->rd_buf[i]=0;
+	drive->rd_buf[1]=0x06;
+	drive->rd_buf[2]=0x02;
+	drive->rd_buf[3]=0xFF;
+	drive->cmd_clear();
+	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
+	drive->cmd[1] = 0x02;
+	drive->cmd[10]= 0x08;
+	if ((drive->err=drive->cmd.transport(WRITE,drive->rd_buf,0x08) ))
+		{ if (!drive->silent) sperror ("PLEXTOR_CLEAR_ASTRATEGY_DB",drive->err); return drive->err;}
+	return 0;
+}
+
+//
 int plextor_modify_autostrategy_db(drive_info* drive, int index, int action)
 {
 	drive->rd_buf[0]=0x00;
@@ -629,7 +801,7 @@ int plextor_modify_autostrategy_db(drive_info* drive, int index, int action)
 	drive->rd_buf[9]=action;
 
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_WR; // 0xE5
+	drive->cmd[0] = PLEXTOR_AS_WR; // 0xE5
 	drive->cmd[1] = 0x02;
 	drive->cmd[10]= 0x0A;
 	if ((drive->err=drive->cmd.transport(WRITE,drive->rd_buf,0x0A) ))
@@ -640,12 +812,9 @@ int plextor_modify_autostrategy_db(drive_info* drive, int index, int action)
 int plextor_create_strategy(drive_info* drive, int mode)
 {
 	int i;
-	if (!drive->silent) printf("AS create: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",
-		(CMD_PLEX_AS_RD) & 0xFF, 4, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-//	return 0;
-
+//	if (!drive->silent) printf("AS create: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",		(CMD_PLEX_AS_RD) & 0xFF, 4, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x04;
 	drive->cmd[2] = mode;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0) ))
@@ -654,29 +823,28 @@ int plextor_create_strategy(drive_info* drive, int mode)
 	if (!drive->silent) printf("AS CRE START...\n");
 
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[10]= 0x12;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
 		{ if (!drive->silent) sperror ("PLEXTOR_CREATE_STRATEGY",drive->err); return drive->err;}
 
 	if (!drive->silent) printf("      AS CRE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i] & 0x0FF); printf("\n");
-
 // Waiting until Strategy is created
 	while (test_unit_ready(drive)) {
 		sleep(1);
 		i++;
 	}
-	printf("Strategy creation time: %d sec",i);
+	printf("Strategy creation time: %d sec\n",i);
 
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[10]= 0x12;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
 		{ if (!drive->silent) sperror ("PLEXTOR_CREATE_STRATEGY_DONE",drive->err); return drive->err;}
 
-	if (!drive->silent) printf(" AS CRE DONE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i]) & 0x0FF; printf("\n");
+//	if (!drive->silent) printf(" AS CRE DONE: "); for (i=0; i<0x12; i++) printf("%02X ", drive->rd_buf[i]) & 0x0FF; printf("\n");
 
 	return 0;
 }
@@ -686,11 +854,11 @@ int plextor_media_check(drive_info* drive, int mode)
 {
 	int i=0;
 	if (!drive->silent) printf("MQCK: %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X\n",
-		(CMD_PLEX_AS_RD) & 0xFF, 1, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+		(PLEXTOR_AS_RD) & 0xFF, 1, mode & 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 //	return 0;
 
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[2] = mode;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0)))
@@ -715,7 +883,7 @@ int plextor_media_check(drive_info* drive, int mode)
 	printf("Media check time: %d sec",i);
 
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_AS_RD;
+	drive->cmd[0] = PLEXTOR_AS_RD;
 	drive->cmd[1] = 0x01;
 	drive->cmd[10]= 0x12;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
@@ -754,7 +922,7 @@ int px755_do_auth(drive_info* drive)
 int px755_get_auth_code(drive_info* drive,unsigned char* auth_code)
 {
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_GET_AUTH;
+	drive->cmd[0] = PLEXTOR_GET_AUTH;
 	drive->cmd[10]= 0x10;
 	if ((drive->err=drive->cmd.transport(READ,auth_code,16) ))
 		{ if (!drive->silent) sperror ("PLEXTOR_PX755_GET_AUTH_CODE",drive->err); return drive->err;}
@@ -768,7 +936,7 @@ int px755_get_auth_code(drive_info* drive,unsigned char* auth_code)
 int px755_send_auth_code(drive_info* drive, unsigned char* auth_code)
 {
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_SEND_AUTH;
+	drive->cmd[0] = PLEXTOR_SEND_AUTH;
 	drive->cmd[1] = 0x01;
 	drive->cmd[2] = 0x01;
 	drive->cmd[10]= 0x10;
@@ -780,7 +948,7 @@ int px755_send_auth_code(drive_info* drive, unsigned char* auth_code)
 int px755_clear_auth_status(drive_info* drive)
 {
 	drive->cmd_clear();
-	drive->cmd[0] = CMD_PLEX_SEND_AUTH;
+	drive->cmd[0] = PLEXTOR_SEND_AUTH;
 	drive->cmd[1] = 0x01;
 	drive->cmd[2] = 0x00;
 	drive->cmd[10]= 0x10;
