@@ -22,7 +22,7 @@
 //#include "transport.hxx"
 #include "qpx_mmc.h"
 
-//#define FEATURE_DEBUG
+#define FEATURE_DEBUG
 
 const unsigned int	bufsz_dev  = 0x0000FF;
 const unsigned int	bufsz_rd   = 0x010000;
@@ -31,7 +31,7 @@ const unsigned int	bufsz_ATIP = 0x000800;
 drive_info::drive_info(){
 }
 
-drive_info::drive_info(char* _device){
+drive_info::drive_info(const char* _device){
 	device=(char*)malloc(bufsz_dev);
 	strcpy(device,_device);
 	rd_buf=(unsigned char*)malloc(bufsz_rd);
@@ -163,6 +163,542 @@ void drive_info::wait_free(){
 }
 */
 
+int print_sense (int err) {
+	char str[128];
+	strcpy(str,"[unknown error]");
+	switch (SK(err)) {
+	case 0x1:
+		switch (ASC(err)) {
+		case 0x0B:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"WARNING"); break;
+        		case 0x01: strcpy(str,"WARNING - SPECIFIED TEMPERATURE EXCEEDED"); break;
+        		case 0x02: strcpy(str,"WARNING - ENCLOSURE DEGRADED"); break;
+
+			default:  sprintf(str,"WARNING, ASCQ=%02X",ASCQ(err)); break;
+			}
+			break;
+		case 0x17:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"RECOVERED DATA WITH NO ERROR CORRECTION APPLIED"); break;
+			case 0x01: strcpy(str,"RECOVERED DATA WITH RETRIES"); break;
+			case 0x02: strcpy(str,"RECOVERED DATA WITH POSITIVE HEAD OFFSET"); break;
+			case 0x03: strcpy(str,"RECOVERED DATA WITH NEGATIVE HEAD OFFSET"); break;
+			case 0x04: strcpy(str,"RECOVERED DATA WITH RETRIES AND/OR CIRC APPLIED"); break;
+			case 0x05: strcpy(str,"RECOVERED DATA USING PREVIOUS SECTOR ID"); break;
+			case 0x07: strcpy(str,"RECOVERED DATA WITHOUT ECC - RECOMMEND REASSIGNMENT"); break;
+			case 0x08: strcpy(str,"RECOVERED DATA WITHOUT ECC - RECOMMEND REWRITE"); break;
+			case 0x09: strcpy(str,"RECOVERED DATA WITHOUT ECC - DATA REWRITTEN"); break;
+
+			default:   strcpy(str,"RECOVERED DATA WITH NO ERROR CORRECTION APPLIED"); break;
+			}
+			break;
+		case 0x18:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"RECOVERED DATA WITH ERROR CORRECTION APPLIED"); break;
+			case 0x01: strcpy(str,"RECOVERED DATA WITH ERROR CORR. & RETRIES APPLIED"); break;
+			case 0x02: strcpy(str,"RECOVERED DATA - DATA AUTO-REALLOCATED"); break;
+			case 0x03: strcpy(str,"RECOVERED DATA WITH CIRC"); break;
+			case 0x04: strcpy(str,"RECOVERED DATA WITH L-EC"); break;
+			case 0x05: strcpy(str,"RECOVERED DATA - RECOMMEND REASSIGNMENT"); break;
+			case 0x06: strcpy(str,"RECOVERED DATA - RECOMMEND REWRITE"); break;
+			case 0x08: strcpy(str,"RECOVERED DATA WITH LINKING"); break;
+
+			default:   strcpy(str,"RECOVERED DATA WITH ERROR CORRECTION APPLIED"); break;
+			}
+			break;
+		case 0x5D:
+			switch (ASCQ(err)) {
+			case 0x01: strcpy(str,"FAILURE PREDICTION THRESHOLD EXCEEDED - Predicted Media failure"); break;
+			case 0x02: strcpy(str,"LOGICAL UNIT FAILURE PREDICTION THRESHOLD EXCEEDED"); break;
+			case 0x03: strcpy(str,"FAILURE PREDICTION THRESHOLD EXCEEDED - Predicted Spare Area Exhaustion"); break;
+			case 0xFF: strcpy(str,"FAILURE PREDICTION THRESHOLD EXCEEDED (FALSE)"); break;
+
+			default:   strcpy(str,"LOGICAL UNIT FAILURE PREDICTION THRESHOLD EXCEEDED"); break;
+			}
+			break;
+		case 0x73:
+			switch (ASCQ(err)) {
+			case 0x01: strcpy(str,"POWER CALIBRATION AREA ALMOST FULL"); break;
+			case 0x06: strcpy(str,"RMA/PMA IS ALMOST FULL"); break;
+			}
+			break;
+		}
+	case 0x2:
+		switch (ASC(err)) {
+		case 0x04:
+			switch (ASCQ(err)) {
+        		case 0x00: strcpy(str,"LOGICAL UNIT NOT READY, CAUSE NOT REPORTABLE"); break;
+        		case 0x01: strcpy(str,"LOGICAL UNIT IS IN PROCESS OF BECOMING READY"); break;
+        		case 0x02: strcpy(str,"LOGICAL UNIT NOT READY, INITIALIZING CMD. REQUIRED"); break;
+        		case 0x03: strcpy(str,"LOGICAL UNIT NOT READY, MANUAL INTERVENTION REQUIRED"); break;
+        		case 0x04: strcpy(str,"LOGICAL UNIT NOT READY, FORMAT IN PROGRESS"); break;
+        		case 0x07: strcpy(str,"LOGICAL UNIT NOT READY, OPERATION IN PROGRESS"); break;
+        		case 0x08: strcpy(str,"LOGICAL UNIT NOT READY, LONG WRITE IN PROGRESS"); break;
+
+        		default:   strcpy(str,"LOGICAL UNIT NOT READY, CAUSE NOT REPORTABLE"); break;
+			}
+			break;
+		case 0x30:
+			switch (ASCQ(err)) {
+       			case 0x00: strcpy(str,"INCOMPATIBLE MEDIUM INSTALLED"); break;
+       			case 0x01: strcpy(str,"CANNOT READ MEDIUM - UNKNOWN FORMAT"); break;
+       			case 0x02: strcpy(str,"CANNOT READ MEDIUM - INCOMPATIBLE FORMAT"); break;
+       			case 0x03: strcpy(str,"CLEANING CARTRIDGE INSTALLED"); break;
+       			case 0x04: strcpy(str,"CANNOT WRITE MEDIUM - UNKNOWN FORMAT"); break;
+       			case 0x05: strcpy(str,"CANNOT WRITE MEDIUM - INCOMPATIBLE FORMAT"); break;
+       			case 0x06: strcpy(str,"CANNOT FORMAT MEDIUM - INCOMPATIBLE MEDIUM"); break;
+       			case 0x07: strcpy(str,"CLEANING FAILURE"); break;
+       			case 0x11: strcpy(str,"CANNOT WRITE MEDIUM - UNSUPPORTED MEDIUM VERSION"); break;
+
+       			default:   strcpy(str,"INCOMPATIBLE MEDIUM INSTALLED"); break;
+			}
+			break;
+		case 0x3A:
+			switch (ASCQ(err)) {
+       			case 0x00: strcpy(str,"MEDIUM NOT PRESENT"); break;
+       			case 0x01: strcpy(str,"MEDIUM NOT PRESENT - TRAY CLOSED"); break;
+       			case 0x02: strcpy(str,"MEDIUM NOT PRESENT - TRAY OPEN"); break;
+
+       			default:   strcpy(str,"MEDIUM NOT PRESENT"); break;
+			}
+			break;
+		case 0x3E: strcpy(str,"LOGICAL UNIT HAS NOT SELF-CONFIGURED YET"); break; /* ASCQ=00: */
+		}
+		break;
+	case 0x3:
+		switch (ASC(err)) {
+		case 0x02: strcpy(str,"NO SEEK COMPLETE"); break; /* ASCQ = 0x00 */
+		case 0x06: strcpy(str,"NO REFERENCE POSITION FOUND"); break;
+		case 0x0C:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"WRITE ERROR"); break;
+			case 0x07: strcpy(str,"WRITE ERROR - RECOVERY NEEDED"); break;
+			case 0x08: strcpy(str,"WRITE ERROR - RECOVERY FAILED"); break;
+			case 0x09: strcpy(str,"WRITE ERROR - LOSS OF STREAMING"); break;
+			case 0x0A: strcpy(str,"WRITE ERROR - PADDING BLOCKS ADDED"); break;
+
+			default:   strcpy(str,"WRITE ERROR"); break;
+			}
+			break;
+		case 0x11:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"UNRECOVERED READ ERROR"); break;
+			case 0x01: strcpy(str,"READ RETRIES EXHAUSTED"); break;
+			case 0x02: strcpy(str,"ERROR TOO LONG TO CORRECT"); break;
+			case 0x05: strcpy(str,"L-EC UNCORRECTABLE ERROR"); break;
+			case 0x06: strcpy(str,"CIRC UNRECOVERED ERROR"); break;
+			case 0x0F: strcpy(str,"ERROR READING UPC/EAN NUMBER"); break;
+			case 0x10: strcpy(str,"ERROR READING ISRC NUMBER"); break;
+
+			default:   strcpy(str,"UNRECOVERED READ ERROR"); break;
+			}
+			break;
+		case 0x15:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"RANDOM POSITIONING ERROR"); break;
+			case 0x01: strcpy(str,"MECHANICAL POSITIONING ERROR"); break;
+			case 0x02: strcpy(str,"POSITIONING ERROR DETECTED BY READ OF MEDIUM"); break;
+
+			default: strcpy(str,"RANDOM POSITIONING ERROR"); break;
+			}
+			break;
+		case 0x31:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"MEDIUM FORMAT CORRUPTED"); break;
+			case 0x01: strcpy(str,"FORMAT COMMAND FAILED"); break;
+			case 0x02: strcpy(str,"ZONED FORMATTING FAILED DUE TO SPARE LINKING"); break;
+
+			default:   strcpy(str,"MEDIUM FORMAT CORRUPTED"); break;
+			}
+			break;
+		case 0x51:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"ERASE FAILURE"); break;
+			case 0x01: strcpy(str,"ERASE FAILURE - INCOMPLETE ERASE OPERATION DETECTED"); break;
+
+			default:   strcpy(str,"ERASE FAILURE"); break;
+			}
+			break;
+		case 0x57: strcpy(str,"UNABLE TO RECOVER TABLE-OF-CONTENTS"); break; /* ASCQ = 00 */
+		case 0x72:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"SESSION FIXATION ERROR"); break;
+			case 0x01: strcpy(str,"SESSION FIXATION ERROR WRITING LEAD-IN"); break;
+			case 0x02: strcpy(str,"SESSION FIXATION ERROR WRITING LEAD-OUT"); break;
+
+			default:   strcpy(str,"SESSION FIXATION ERROR"); break;
+			}
+			break;
+		case 0x73:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"CD CONTROL ERROR"); break;
+			case 0x02: strcpy(str,"POWER CALIBRATION AREA IS FULL"); break;
+			case 0x03: strcpy(str,"POWER CALIBRATION AREA ERROR"); break;
+			case 0x04: strcpy(str,"PROGRAM MEMORY AREA UPDATE FAILURE"); break;
+			case 0x05: strcpy(str,"PROGRAM MEMORY AREA IS FULL"); break;
+
+			default:   strcpy(str,"CD CONTROL ERROR"); break;
+			}
+			break;
+		}
+		break;
+	case 0x4:
+		switch (ASC(err)) {
+		case 0x00: strcpy(str,"CLEANING REQUESTED"); break;  /* ASCQ = 0x17 */
+		case 0x05: strcpy(str,"LOGICAL UNIT DOES NOT RESPOND TO SELECTION"); break; /* ASCQ = 0x00 */
+		case 0x08:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"LOGICAL UNIT COMMUNICATION FAILURE"); break;
+			case 0x01: strcpy(str,"LOGICAL UNIT COMMUNICATION TIMEOUT"); break;
+			case 0x02: strcpy(str,"LOGICAL UNIT COMMUNICATION PARITY ERROR"); break;
+			case 0x03: strcpy(str,"LOGICAL UNIT COMMUNICATION CRC ERROR (ULTRA-DMA/32)"); break;
+			}
+			break;
+		case 0x09:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"TRACK FOLLOWING ERROR"); break;
+			case 0x01: strcpy(str,"TRACKING SERVO FAILURE"); break;
+			case 0x02: strcpy(str,"FOCUS SERVO FAILURE"); break;
+			case 0x03: strcpy(str,"SPINDLE SERVO FAILURE"); break;
+			case 0x04: strcpy(str,"HEAD SELECT FAULT"); break;
+
+			default:   strcpy(str,"TRACKING ERROR"); break;
+			}
+			break;
+		case 0x15:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"RANDOM POSITIONING ERROR"); break;
+			case 0x01: strcpy(str,"MECHANICAL POSITIONING ERROR"); break;
+
+			default:   strcpy(str,"RANDOM POSITIONING ERROR"); break;
+			}
+			break;
+		case 0x1B: strcpy(str,"SYNCHRONOUS DATA TRANSFER ERROR"); break; /* ASCQ = 0x00 */
+		case 0x3B: strcpy(str,"MECHANICAL POSITIONING OR CHANGER ERROR"); break; /* ASCQ = 0x16 */
+		case 0x3E:
+			switch (ASCQ(err)) {
+			case 0x01: strcpy(str,"LOGICAL UNIT FAILURE"); break;
+			case 0x02: strcpy(str,"TIMEOUT ON LOGICAL UNIT"); break;
+
+			default:   strcpy(str,"LOGICAL UNIT FAILURE"); break;
+			}
+			break;
+		case 0x40: strcpy(str,"DIAGNOSTIC FAILURE ON COMPONENT NN (80H-FFH)"); break;
+		case 0x44: strcpy(str,"INTERNAL TARGET FAILURE"); break;
+		case 0x46: strcpy(str,"UNSUCCESSFUL SOFT RESET"); break;
+		case 0x47: strcpy(str,"SCSI PARITY ERROR"); break;
+		case 0x4A: strcpy(str,"COMMAND PHASE ERROR"); break;
+		case 0x4B: strcpy(str,"DATA PHASE ERROR"); break;
+		case 0x4C: strcpy(str,"LOGICAL UNIT FAILED SELF-CONFIGURATION"); break;
+		case 0x53: strcpy(str,"MEDIA LOAD OR EJECT FAILED"); break;
+		case 0x65: strcpy(str,"VOLTAGE FAULT"); break;
+		}
+		break;
+	case 0x5:
+		switch (ASC(err)) {
+		case 0x07: strcpy(str,"MULTIPLE PERIPHERAL DEVICES SELECTED"); break; /* ASCQ = 0x00 */
+		case 0x1A: strcpy(str,"PARAMETER LIST LENGTH ERROR"); break; /* ASCQ = 0x00 */
+		case 0x20: strcpy(str,"INVALID COMMAND OPERATION CODE"); break; /* ASCQ = 0x00 */
+		case 0x21:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"LOGICAL BLOCK ADDRESS OUT OF RANGE"); break;
+			case 0x01: strcpy(str,"INVALID ELEMENT ADDRESS"); break;
+			case 0x02: strcpy(str,"INVALID ADDRESS FOR WRITE"); break;
+
+			default:   strcpy(str,"LOGICAL BLOCK ADDRESS OUT OF RANGE"); break;
+			}
+			break;
+		case 0x24: strcpy(str,"INVALID FIELD IN CDB"); break;
+		case 0x25: strcpy(str,"LOGICAL UNIT NOT SUPPORTED"); break;
+		case 0x26:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"INVALID FIELD IN PARAMETER LIST"); break;
+			case 0x01: strcpy(str,"PARAMETER NOT SUPPORTED"); break;
+			case 0x02: strcpy(str,"PARAMETER VALUE INVALID"); break;
+			case 0x03: strcpy(str,"THRESHOLD PARAMETERS NOT SUPPORTED"); break;
+			}
+			break;
+		case 0x2B: strcpy(str,"COPY CANNOT EXECUTE SINCE INITIATOR CANNOT DISCONNECT"); break; /* ASCQ = 0x00 */
+		case 0x2C:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"COMMAND SEQUENCE ERROR"); break;
+			case 0x03: strcpy(str,"CURRENT PROGRAM AREA IS NOT EMPTY"); break;
+			case 0x04: strcpy(str,"CURRENT PROGRAM AREA IS EMPTY"); break;
+			}
+			break;
+		case 0x30:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"INCOMPATIBLE MEDIUM INSTALLED"); break;
+			case 0x01: strcpy(str,"CANNOT READ MEDIUM - UNKNOWN FORMAT"); break;
+			case 0x02: strcpy(str,"CANNOT READ MEDIUM - INCOMPATIBLE FORMAT"); break;
+			case 0x03: strcpy(str,"CLEANING CARTRIDGE INSTALLED"); break;
+			case 0x04: strcpy(str,"CANNOT WRITE MEDIUM - UNKNOWN FORMAT"); break;
+			case 0x05: strcpy(str,"CANNOT WRITE MEDIUM - INCOMPATIBLE FORMAT"); break;
+			case 0x06: strcpy(str,"CANNOT FORMAT MEDIUM - INCOMPATIBLE MEDIUM"); break;
+			case 0x07: strcpy(str,"CLEANING FAILURE"); break;
+			case 0x08: strcpy(str,"CANNOT WRITE - APPLICATION CODE MISMATCH"); break;
+			case 0x09: strcpy(str,"CURRENT SESSION NOT FIXATED FOR APPEND"); break;
+			case 0x10: strcpy(str,"MEDIUM NOT FORMATTED"); break;
+			}
+			break;
+		case 0x39: strcpy(str,"SAVING PARAMETERS NOT SUPPORTED"); break;  /* ASCQ = 0x00 */
+		case 0x3D: strcpy(str,"INVALID BITS IN IDENTIFY MESSAGE"); break; /* ASCQ = 0x00 */
+		case 0x43: strcpy(str,"MESSAGE ERROR"); break; /* ASCQ = 0x00 */
+		case 0x53: strcpy(str,"MEDIUM REMOVAL PREVENTED"); break; /* ASCQ = 0x02 */
+		case 0x64:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"ILLEGAL MODE FOR THIS TRACK"); break;
+			case 0x01: strcpy(str,"INVALID PACKET SIZE"); break;
+			}
+			break;
+		case 0x6F:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"COPY PROTECTION KEY EXCHANGE FAILURE - AUTHENTICATION FAILURE"); break;
+			case 0x01: strcpy(str,"COPY PROTECTION KEY EXCHANGE FAILURE - KEY NOT PRESENT"); break;
+			case 0x02: strcpy(str,"COPY PROTECTION KEY EXCHANGE FAILURE - KEY NOT ESTABLISHED"); break;
+			case 0x03: strcpy(str,"READ OF SCRAMBLED SECTOR WITHOUT AUTHENTICATION"); break;
+			case 0x04: strcpy(str,"MEDIA REGION CODE IS MISMATCHED TO LOGICAL UNIT REGION"); break;
+			case 0x05: strcpy(str,"LOGICAL UNIT REGION MUST BE PERMANENT/REGION RESET COUNT ERROR"); break;
+			}
+			break;
+		case 0x72:
+			switch (ASCQ(err)) {
+			case 0x03: strcpy(str,"SESSION FIXATION ERROR . INCOMPLETE TRACK IN SESSION"); break;
+			case 0x04: strcpy(str,"EMPTY OR PARTIALLY WRITTEN RESERVED TRACK"); break;
+			case 0x05: strcpy(str,"NO MORE TRACK RESERVATIONS ALLOWED"); break;
+			}
+			break;
+		}
+		break;
+	case 0x6:
+		switch (ASC(err)) {
+		case 0x28:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"NOT READY TO READY CHANGE, MEDIUM MAY HAVE CHANGED"); break;
+			case 0x01: strcpy(str,"IMPORT OR EXPORT ELEMENT ACCESSED"); break;
+			}
+			break;
+		case 0x29:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"POWER ON, RESET, OR BUS DEVICE RESET OCCURRED"); break;
+			case 0x01: strcpy(str,"POWER ON OCCURRED"); break;
+			case 0x02: strcpy(str,"BUS RESET OCCURRED"); break;
+			case 0x03: strcpy(str,"BUS DEVICE RESET FUNCTION OCCURRED"); break;
+			case 0x04: strcpy(str,"DEVICE INTERNAL RESET"); break;
+			}
+			break;
+		case 0x2A:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"PARAMETERS CHANGED"); break;
+			case 0x01: strcpy(str,"MODE PARAMETERS CHANGED"); break;
+			case 0x02: strcpy(str,"LOG PARAMETERS CHANGED"); break;
+			case 0x03: strcpy(str,"RESERVATIONS PREEMPTED"); break;
+			}
+			break;
+		case 0x2E: strcpy(str,"INSUFFICIENT TIME FOR OPERATION"); break;
+		case 0x2F: strcpy(str,"COMMANDS CLEARED BY ANOTHER INITIATOR"); break;
+		case 0x3B:
+			switch (ASCQ(err)) {
+			case 0x0D: strcpy(str,"MEDIUM DESTINATION ELEMENT FULL"); break;
+			case 0x0E: strcpy(str,"MEDIUM SOURCE ELEMENT EMPTY"); break;
+			case 0x0F: strcpy(str,"END OF MEDIUM REACHED"); break;
+			case 0x11: strcpy(str,"MEDIUM MAGAZINE NOT ACCESSIBLE"); break;
+			case 0x12: strcpy(str,"MEDIUM MAGAZINE REMOVED"); break;
+			case 0x13: strcpy(str,"MEDIUM MAGAZINE INSERTED"); break;
+			case 0x14: strcpy(str,"MEDIUM MAGAZINE LOCKED"); break;
+			case 0x15: strcpy(str,"MEDIUM MAGAZINE UNLOCKED"); break;
+			}
+			break;
+		case 0x3F:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"TARGET OPERATING CONDITIONS HAVE CHANGED"); break;
+			case 0x01: strcpy(str,"MICROCODE HAS BEEN CHANGED"); break;
+			case 0x02: strcpy(str,"CHANGED OPERATING DEFINITION"); break;
+			case 0x03: strcpy(str,"INQUIRY DATA HAS CHANGED"); break;
+			}
+			break;
+		case 0x5A:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"OPERATOR REQUEST OR STATE CHANGE INPUT"); break;
+			case 0x01: strcpy(str,"OPERATOR MEDIUM REMOVAL REQUEST"); break;
+			case 0x02: strcpy(str,"OPERATOR SELECTED WRITE PROTECT"); break;
+			case 0x03: strcpy(str,"OPERATOR SELECTED WRITE PERMIT"); break;
+			}
+			break;
+		case 0x5B:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"LOG EXCEPTION"); break;
+			case 0x01: strcpy(str,"THRESHOLD CONDITION MET"); break;
+			case 0x02: strcpy(str,"LOG COUNTER AT MAXIMUM"); break;
+			case 0x03: strcpy(str,"LOG LIST CODES EXHAUSTED"); break;
+			}
+			break;
+		case 0x5E:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"LOW POWER CONDITION ON"); break;
+			case 0x01: strcpy(str,"IDLE CONDITION ACTIVATED BY TIMER"); break;
+			case 0x02: strcpy(str,"STANDBY CONDITION ACTIVATED BY TIMER"); break;
+			case 0x03: strcpy(str,"IDLE CONDITION ACTIVATED BY COMMAND"); break;
+			case 0x04: strcpy(str,"STANDBY CONDITION ACTIVATED BY COMMAND"); break;
+			}
+			break;
+		}
+		break;
+	case 0x7:
+		switch (ASC(err)) {
+		case 0x27:
+			switch (ASCQ(err)) {
+			case 0x00: strcpy(str,"WRITE PROTECTED"); break;
+			case 0x01: strcpy(str,"HARDWARE WRITE PROTECTED"); break;
+			case 0x02: strcpy(str,"LOGICAL UNIT SOFTWARE WRITE PROTECTED"); break;
+			case 0x03: strcpy(str,"ASSOCIATED WRITE PROTECT"); break;
+			case 0x04: strcpy(str,"PERSISTENT WRITE PROTECT"); break;
+			case 0x05: strcpy(str,"PERMANENT WRITE PROTECT"); break;
+			case 0x06: strcpy(str,"CONDITIONAL WRITE PROTECT"); break;
+
+			default:   strcpy(str,"WRITE PROTECTED"); break;
+			}
+			break;
+		}
+		break;
+	case 0x8: strcpy(str,"BLANK CHECK"); break;
+	case 0xB:
+		switch (ASC(err)) {
+		case 0x00: strcpy(str,"I/O PROCESS TERMINATED"); break; /* ASCQ = 06 */
+        	case 0x11: strcpy(str,"READ ERROR - LOSS OF STREAMING"); break; /* ASCQ = 11 */
+        	case 0x45: strcpy(str,"SELECT OR RESELECT FAILURE"); break; /* ASCQ = 00 */
+        	case 0x48: strcpy(str,"INITIATOR DETECTED ERROR MESSAGE RECEIVED"); break; /* ASCQ = 00 */
+        	case 0x49: strcpy(str,"INVALID MESSAGE ERROR"); break; /* ASCQ = 00 */
+        	case 0x4D: strcpy(str,"TAGGED OVERLAPPED COMMANDS (NN = QUEUE TAG)"); break; /* ASCQ = xx */
+		}
+		break;
+	}
+	printf("[%05X]  %s", err, str);
+	return 0;
+}
+
+int print_opcode (unsigned char opcode) {
+	char str[128];
+	switch (opcode) {
+
+//	case 0x00: strcpy(str,"      "); break;
+
+
+		case 0x00: strcpy(str,"SPC_TEST_UNIT_READY                           "); break;
+		case 0x03: strcpy(str,"SPC_REQUEST_SENSE                             "); break;
+		case 0x04: strcpy(str,"MMC_FORMAT_UNIT                               "); break;
+
+		case 0x12: strcpy(str,"SPC_INQUIRY                                   "); break;
+		case 0x15: strcpy(str,"MMC_MODE_SELECT6                              "); break;
+		case 0x16: strcpy(str,"SPC_RESERVE_6                                 "); break;
+		case 0x17: strcpy(str,"SPC_RELEASE_6                                 "); break;
+		case 0x1A: strcpy(str,"MMC_MODE_SENSE6                               "); break;
+		case 0x1B: strcpy(str,"MMC_START_STOP_UNIT                           "); break;
+		case 0x1C: strcpy(str,"SPC_RECEIVE_DIAGNOSTIC_RESULTS                "); break;
+		case 0x1D: strcpy(str,"SPC_SEND_DIAGNOSTIC                           "); break;
+		case 0x1E: strcpy(str,"SPC_PREVENT_ALLOW_MEDIUM_REMIVAL              "); break;
+
+
+		case 0x23: strcpy(str,"MMC_READ_FORMAT_CAPACITIES                    "); break;
+		case 0x25: strcpy(str,"MMC_READ_RECORDED_CAPACITY                    "); break;
+		case 0x28: strcpy(str,"MMC_READ                                      "); break;
+		case 0x2A: strcpy(str,"MMC_WRITE                                     "); break;
+		case 0x2B: strcpy(str,"MMC_SEEK                                      "); break;
+		case 0x2E: strcpy(str,"MMC_WRITE_AND_VERIFY                          "); break;
+		case 0x2F: strcpy(str,"MMC_VERIFY                                    "); break;
+
+		case 0x35: strcpy(str,"MMC_SYNC_CACHE                                "); break;
+		case 0x3B: strcpy(str,"SPC_WRITE_BUFFER                              "); break;
+		case 0x3C: strcpy(str,"SPC_READ_BUFFER                               "); break;
+
+		case 0x42: strcpy(str,"MMC_READ_SUB_CHANNEL                          "); break;
+		case 0x43: strcpy(str,"MMC_READ_TOC_PMA_ATIP                         "); break;
+		case 0x44: strcpy(str,"MMC_READ_HEADER                               "); break;
+		case 0x45: strcpy(str,"MMC_PLAY_AUDIO                                "); break;
+		case 0x46: strcpy(str,"MMC_GET_CONFIGURATION                         "); break;
+		case 0x47: strcpy(str,"MMC_PLAY_AUDIO_MSF                            "); break;
+		case 0x4A: strcpy(str,"MMC_GET_EVENT_STATUS_NOTIFICATION             "); break;
+		case 0x4B: strcpy(str,"MMC_PAUSE_RESUME                              "); break;
+		case 0x4C: strcpy(str,"SPC_LOG_SELECT                                "); break;
+		case 0x4D: strcpy(str,"SPC_LOG_SENSE                                 "); break;
+		case 0x4E: strcpy(str,"MMC_STOP_PLAY_SCAN                            "); break;
+
+		case 0x51: strcpy(str,"MMC_READ_DISC_INFORMATION                     "); break;
+		case 0x52: strcpy(str,"MMC_READ_TRACK_INFORMATION                    "); break;
+		case 0x53: strcpy(str,"MMC_RESERVE_TRACK                             "); break;
+		case 0x54: strcpy(str,"MMC_SEND_OPC_INFORMATION                      "); break;
+		case 0x55: strcpy(str,"MMC_MODE_SELECT10                             "); break;
+		case 0x56: strcpy(str,"SPC_RESERVE_10                                "); break;
+		case 0x57: strcpy(str,"SPC_RELEASE_10                                "); break;
+		case 0x58: strcpy(str,"MMC_REPAIR_TRACK                              "); break;
+		case 0x59: strcpy(str,"MMC_READ_MASTER_CUE                           "); break;
+		case 0x5A: strcpy(str,"MMC_MODE_SENSE10                              "); break;
+		case 0x5B: strcpy(str,"MMC_CLOSE_TRACK_SESSION                       "); break;
+		case 0x5C: strcpy(str,"MMC_READ_BUFFER_CAPACITY                      "); break;
+		case 0x5D: strcpy(str,"MMC_SEND_CUE_SHEET                            "); break;
+		case 0x5E: strcpy(str,"SPC_PERSISTENT_RESERVE_IN                     "); break;
+		case 0x5F: strcpy(str,"SPC_PERSISTENT_RESERVE_OUT                    "); break;
+
+		case 0x83: strcpy(str,"SPC_EXTENDED_COPY                             "); break;
+		case 0x84: strcpy(str,"SPC_RECEIVE_COPY_RESULTS                      "); break;
+		case 0x86: strcpy(str,"SPC_ACCESS_CONTROL_IN                         "); break;
+		case 0x87: strcpy(str,"SPC_ACCESS_CONTROL_OUT                        "); break;
+		case 0x8C: strcpy(str,"SPC_READ_ATTRIBUTE                            "); break;
+		case 0x8D: strcpy(str,"SPC_WRITE_ATTRIBUTE                           "); break;
+
+		case 0xA0: strcpy(str,"SPC_REPORT_LUNS                               "); break;
+		case 0xA1: strcpy(str,"MMC_BLANK                                     "); break;
+		case 0xA2: strcpy(str,"MMC_SEND_EVENT     / SPC_SECURITY_PROTOCOL_IN "); break;
+		case 0xA3: strcpy(str,"MMC_SEND_KEY                                  "); break;
+		case 0xA4: strcpy(str,"MMC_REPORT_KEY                                "); break;
+		case 0xA5: strcpy(str,"MMC_PLAY_AUDIO_12                             "); break;
+		case 0xA6: strcpy(str,"MMC_LOAD_UNLOAD                               "); break;
+		case 0xA7: strcpy(str,"MMC_SET_READ_AHEAD / SMC_MOVE_MEDIUM_ATTACHED "); break;
+		case 0xA8: strcpy(str,"MMC_READ_DVD                                  "); break;
+		case 0xAC: strcpy(str,"MMC_GET_PERFORMANCE                           "); break;
+		case 0xAD: strcpy(str,"MMC_READ_DVD_STRUCTURE                        "); break;
+
+		case 0xB4: strcpy(str,"SMC_READ_ELEMENT_STATUS_ATTACHED              "); break;
+		case 0xB5: strcpy(str,"SPC_SECURITY_PROTOCOL_OUT                     "); break;
+		case 0xB6: strcpy(str,"MMC_SET_STREAMING                             "); break;
+		case 0xB9: strcpy(str,"MMC_READ_CD_MSF                               "); break;
+		case 0xBA: strcpy(str,"MMC_SCAN                                      "); break;
+		case 0xBB: strcpy(str,"MMC_SET_SPEED                                 "); break;
+		case 0xBC: strcpy(str,"MMC_PLAY_CD          / SCC_SPARE_IN           "); break;
+		case 0xBD: strcpy(str,"MMC_MECHANISM_STATUS / SCC_SPARE_OUT          "); break;
+		case 0xBE: strcpy(str,"MMC_READ_CD          / SCC_VOLUME_SET_IN      "); break;
+		case 0xBF: strcpy(str,"SCC_VOLUME_SET_OUT                            "); break;
+
+		case 0xD4: strcpy(str,"PLEXTOR_GET_AUTH                              "); break;
+		case 0xD5: strcpy(str,"PLEXTOR_SEND_AUTH                             "); break;
+
+//const char  = 0xD8;
+
+		case 0xE3: strcpy(str,"PLEXTOR_ERASER                                "); break;
+		case 0xE4: strcpy(str,"PLEXTOR_AS_RD                                 "); break;
+		case 0xE5: strcpy(str,"PLEXTOR_AS_WR                                 "); break;
+
+		case 0xE7: strcpy(str,"SBC_FLUSH_CACHE                               "); break;
+
+		case 0xE9: strcpy(str,"PLEXTOR_MODE                                  "); break;
+		case 0xEA: strcpy(str,"PLEXTOR_QCHECK                                "); break;
+		case 0xEB: strcpy(str,"PLEXTOR_PREC_SPD                              "); break;
+		case 0xED: strcpy(str,"PLEXTOR_MODE2                                 "); break;
+
+		case 0xEE: strcpy(str,"PLEXTOR_RESTART                               "); break;
+//		case 0xEF: strcpy(str,"PLEXTOR     REBOOT ???                        "); break;
+
+		case 0xF1: strcpy(str,"PLEXTOR_EEPROM_READ                           "); break;
+
+		case 0xF3: strcpy(str,"PLEXTOR_SCAN_TA_FETE                          "); break;
+		case 0xF5: strcpy(str,"PLEXTOR_FETE_READOUT                          "); break;
+		default:   strcpy(str,"*unknown*                                     ");
+	}
+	printf("[%02X]  %s", opcode, str);
+	return 0;
+}
+
 void spinup(drive_info* drive, unsigned char secs) {
 	long st, et;
 	char use_readcd = 0;
@@ -190,7 +726,7 @@ int inquiry(drive_info* drive) {
 	char	data[36];	
 	if (drive->mmc == -1) return ERR_NO_DEV;
 	drive->cmd_clear();
-	drive->cmd[0] = MMC_INQUIRY;
+	drive->cmd[0] = SPC_INQUIRY;
 	drive->cmd[4] = 36;
 	drive->cmd[5] = 0;
 	drive->err=drive->cmd.transport(READ,data,36);
@@ -206,28 +742,33 @@ int inquiry(drive_info* drive) {
 	return 0;
 }
 
+
 int test_unit_ready(drive_info* drive) {
 	drive->cmd_clear();
-	drive->cmd[0] = MMC_TEST_UNIT_READY;
-	if ((drive->err=drive->cmd.transport(NONE, NULL, 0) )) return drive->err;
-	return 0;
+	drive->cmd[0] = SPC_TEST_UNIT_READY;
+	return drive->err=drive->cmd.transport(NONE, NULL, 0);
 }
 
-int wait_unit_ready(drive_info* drive, int secs) {
+int wait_unit_ready(drive_info* drive, int secs, bool need_media) {
 	long st, et;
 	st = getmsecs();
 	for ( et = getmsecs(); (et-st) < ((long)secs*1000); et = getmsecs() ) {
-//		printf("Remaining: %.3f sec...\r", secs - ((et-st) / 1000.0));
+		printf("Remaining: %.3f sec...\n", secs - ((et-st) / 1000.0));
 		if (!test_unit_ready(drive)) return 0;
-		sleep(1);
+		if (!need_media) {
+			if (drive->err == 0x23A01) return 0;
+			if (drive->err == 0x23A02) return 0;
+		}
+		usleep(100000);
 //		i++;
 	}
-	printf("wait_unit_ready(): Time Out (%ds)", secs);
-	return 1;
+	printf("wait_unit_ready(): Time Out (%ds)\n", secs);
+	return 0;
 }
+
 int request_sense(drive_info* drive, char add){
 	drive->cmd_clear();
-	drive->cmd[0]= MMC_REQUEST_SENSE;
+	drive->cmd[0]= SPC_REQUEST_SENSE;
 	drive->cmd[4]= 0x12+add;
 	if ((drive->err=drive->cmd.transport(READ,drive->rd_buf,0x12) ))
 		{sperror ("REQUEST_SENSE",drive->err); return (drive->err); }
@@ -249,7 +790,7 @@ int mode_sense6(drive_info* drive, int page, int page_control, short dest_len) {
 
 int mode_sense(drive_info* drive, int page, int page_control, int dest_len) {
 	drive->cmd_clear();
-	drive->cmd[0]=MMC_MODE_SENSE10;
+	drive->cmd[0]=MMC_MODE_SENSE_10;
 	drive->cmd[2]=page_control << 6 | page;
 	drive->cmd[7]=(dest_len >> 8) & 0xFF;
 	drive->cmd[8]=dest_len & 0xFF;
@@ -272,7 +813,7 @@ int mode_select6(drive_info* drive, short dest_len) {
 
 int mode_select(drive_info* drive, int dest_len) {
 	drive->cmd_clear();
-	drive->cmd[0]=MMC_MODE_SELECT10;
+	drive->cmd[0]=MMC_MODE_SELECT_10;
 	drive->cmd[1]= 0x10;
 	drive->cmd[7]=(dest_len >> 8) & 0xFF;
 	drive->cmd[8]=dest_len & 0xFF;
@@ -345,7 +886,7 @@ int get_features_list(drive_info* drive) {
 	unsigned int len, i, ii;
 	unsigned short fn;
 	unsigned char  fv;
-	unsigned char  fl;
+	unsigned int  fl;
 	if (!drive->silent) printf("\n** Reading supported features...\n");
 	if (get_configuration(drive, 0 , &len, NULL, 0)) return 1;
 #if 0	
@@ -356,13 +897,16 @@ int get_features_list(drive_info* drive) {
 	for (i=8; i<len; i+=fl) {
 		fn = swap2u (drive->rd_buf+i);
 		fv = drive->rd_buf[i+2];
-		fl = drive->rd_buf[i+3];
+		fl = (unsigned int) drive->rd_buf[i+3];
 		ii = 0;
 		while ((FEATURES[ii].id != fn) && (FEATURES[ii].id < 0xFFFF)) ii++;
 		if (!drive->silent) {
 			printf("Feature: 0x%04X, ver %2X [%s]", fn, fv, FEATURES[ii].name);
 #ifdef FEATURE_DEBUG
-			for (ii=0; ii<(fl+4); ii++) { if (!(ii%32)) printf("\n"); printf(" %02X",drive->rd_buf[i+ii] & 0xFF); }
+			for (ii=0; ii<(fl+4); ii++) {
+				if (!(ii%32)) printf("\n");
+				printf(" %02X",drive->rd_buf[i+ii] & 0xFF);
+			}
 #endif
 			printf("\n");
 		}
@@ -404,12 +948,12 @@ int get_profiles_list(drive_info* drive) {
 				drive->wr_capabilities |= DEVICE_DVD_RW;
 				break;
 			case 0x15:						// DVD-R DL Sequential
-				drive->rd_capabilities |= (DEVICE_DVD_R_DL);
-				drive->wr_capabilities |= (DEVICE_DVD_R_DL);
+				drive->rd_capabilities |= DEVICE_DVD_R_DL;
+				drive->wr_capabilities |= DEVICE_DVD_R_DL;
 				break;
 			case 0x16:						// DVD-R DL Layer Jump
-				drive->rd_capabilities |= (DEVICE_DVD_R_DL);
-				drive->wr_capabilities |= (DEVICE_DVD_R_DL);
+				drive->rd_capabilities |= DEVICE_DVD_R_DL;
+				drive->wr_capabilities |= DEVICE_DVD_R_DL;
 				break;
 			case 0x1A:						// DVD+RW
 				drive->rd_capabilities |= DEVICE_DVD_PLUS_RW;
@@ -487,11 +1031,13 @@ void detect_capabilities(drive_info* drive){
 		if (len >= 0x08) {
 			if ((drive->rd_buf[10] >> 2) > 0) drive->mmc=5;
 			if (len >= 0x0C) {
-			    if (drive->rd_buf[12]&0x01) {
+//			    if (drive->rd_buf[12]&0x01) {
+			    if (drive->rd_buf[14]&0x01) {
 					drive->rd_capabilities|=DEVICE_DVD_RW;
 				//	drive->rd_capabilities|=DEVICE_DVD_R_DL;
 				}
-				if (drive->rd_buf[14]&0x01) drive->rd_capabilities|=DEVICE_DVD_R_DL;
+//			    if (drive->rd_buf[14]&0x01) drive->rd_capabilities|=DEVICE_DVD_R_DL;
+			    if (drive->rd_buf[12]&0x01) drive->rd_capabilities|=DEVICE_DVD_R_DL;
 			}
 			get_configuration(drive, FEATURE_DVD_CPRM , &len, NULL);
 			if (len >= 0x08) drive->capabilities|=CAP_DVD_CPRM;
@@ -991,12 +1537,12 @@ int read_mediaid_cd(drive_info* drive)
 }
 
 int determine_disc_type(drive_info* drive) {
-	int current = 0;
+//	int current = 0;
 //	int i=0;
 	drive->media.disc_type = DISC_NODISC;
 // 	drive->media.type = Media_NoMedia;
 	if (drive->mmc>1) {
-		get_configuration(drive, FEATURE_PROFILE_LIST, NULL, &current);
+		get_configuration(drive, FEATURE_PROFILE_LIST, NULL, 0);
 		switch (drive->rd_buf[7]) {
 			case 0: drive->media.disc_type = DISC_NODISC; break;
 			case PROFILE_CD_ROM:		drive->media.disc_type = DISC_CDROM; break;
@@ -1307,9 +1853,9 @@ int get_lock(drive_info* drive){
 int set_lock(drive_info* drive){
 	int lock;
 //	printf("--- %slocking disc...\n",(drive->parms.status & STATUS_LOCK) ? "" : "UN");
-	if (drive->parms.status & STATUS_LOCK) lock=1; else lock=0;	
+	if (drive->parms.status & STATUS_LOCK) lock=1; else lock=0;
 	drive->cmd_clear();
-	drive->cmd[0]=MMC_PREVENT_ALLOW_MEDIUM_REMIVAL;
+	drive->cmd[0]=SPC_PREVENT_ALLOW_MEDIUM_REMIVAL;
 	drive->cmd[4]=lock;
 	if ((drive->err=drive->cmd.transport(NONE, NULL, 0)))
 		{ sperror ("SET_LOCK",drive->err); get_lock(drive); return (drive->err); }
